SystemOrganization addCategory: #'Spirit-Core'!

MooseModel subclass: #SpiritModel
	instanceVariableNames: 'spiritFeatures hotSpotsDetected spiritScenarios locHistory bethaValues systemReturnsCache percentagesOfChanges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 7/10/2012 12:14'!
addFeature: aFeature
	spiritFeatures add:aFeature.! !

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 7/10/2012 12:14'!
addScenario: aScenario
	spiritScenarios add: aScenario.! !

!SpiritModel methodsFor: 'BetaCalculation' stamp: 'SantiagoVidal 7/14/2012 14:18'!
calculateBetaValueFor: aFAMIXClass
"Obtener valor de LOC por version
calcular retornos (para clases y sistema): LOCMasReciente/LOCAnterior -1 
Cacular el beta: covarianza entre los retornos de la clase y el sistema dividido la variaznza del sistema
la covarianza en excell es COVAR y la varianza es VARP"
	|beta classReturns subArraySystemReturns systemVariance|
	classReturns:=self calculateReturnsFor: aFAMIXClass.
	subArraySystemReturns:=(self systemReturns)copyFrom: (((self systemReturns) size)-(classReturns size)+1) to: (self systemReturns size). 
	systemVariance:=(SpiritMathUtil variance: subArraySystemReturns).
	(systemVariance =0)ifTrue: [beta:=0] ifFalse: [
		beta:=(SpiritMathUtil covarianceBetween: classReturns and: subArraySystemReturns)/systemVariance.
	].
	^beta! !

!SpiritModel methodsFor: 'BetaCalculation' stamp: 'SantiagoVidal 7/16/2012 10:11'!
calculateBetaValues
	"Aca hay que actualizar los datos. Calcular los beta para cada clase que haya en el modelo (es decir, ultima version del sistema). ""No olvidar la version actual!!!!"
	
	|classes|
	classes:=self allClasses select: [:each | each isStub not].
	systemReturnsCache:=nil.
	bethaValues:=Dictionary new. "Hashtable con classes en los key y valor de betha en el value"
	percentagesOfChanges:=Dictionary new.
	classes do:[:aFAMIXClass|
		"(aFAMIXClass name='FamixRequestor')ifTrue: [self halt]."
		bethaValues at: aFAMIXClass put: (self calculateBetaValueFor: aFAMIXClass) abs.
		percentagesOfChanges at: aFAMIXClass put: (self calculatePercentageOfChangeFor:aFAMIXClass).
	].

	self updateHotSpotsSeverity.! !

!SpiritModel methodsFor: 'BetaCalculation' stamp: 'SantiagoVidal 7/16/2012 09:32'!
calculatePercentageOfChangeFor: aFAMIXClass
	
	| changesInClass historyLengh|
	changesInClass:=((self calculateReturnsFor: aFAMIXClass) select: [:aN| (aN=0)not])size.
	historyLengh:=(locHistory size) +1.
	^(changesInClass asFloat)/(historyLengh asFloat).! !

!SpiritModel methodsFor: 'BetaCalculation' stamp: 'SantiagoVidal 7/14/2012 11:31'!
calculateReturnsFor: aFAMIXClass
	|ret|
	ret:=OrderedCollection new.
	locHistory do:[:aDictionary|
		|locValue|
		locValue:=aDictionary at: (aFAMIXClass asString) ifAbsent:[nil].
		(locValue=nil)ifFalse:[ret add:locValue].
	].
	ret add: aFAMIXClass numberOfLinesOfCode."Agrego la version actual"
	^self createReturnsFrom:ret.! !

!SpiritModel methodsFor: 'BetaCalculation' stamp: 'SantiagoVidal 7/14/2012 11:55'!
createReturnsFrom: aCollectionOfValues
	|returns |
	returns:=OrderedCollection new.
	
	2 to: (aCollectionOfValues size) do:[:aValue| 
		((aCollectionOfValues at:(aValue-1))=0)ifTrue: [returns add:0] ifFalse: [
			returns add: (((aCollectionOfValues at:(aValue)) asFloat)/(aCollectionOfValues at:(aValue-1))asFloat)-1.
		].
	].

	^returns.! !

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 09:33'!
detectHotSpots
	|aDictionary|
	aDictionary:=SPAnalizer new analyseMooseModel: self.
	self hotSpotsDetected: aDictionary.! !

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 7/5/2012 16:35'!
hotSpotsDetected
	(hotSpotsDetected =nil) ifTrue:[self detectHotSpots].
	^hotSpotsDetected! !

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 09:31'!
hotSpotsDetected: aDictionary
	hotSpotsDetected:=aDictionary! !

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 7/16/2012 09:17'!
initialize
	super initialize.
	spiritFeatures:=OrderedCollection new.
	spiritScenarios:=OrderedCollection new.
	hotSpotsDetected:=nil.
	bethaValues:=Dictionary new. "Hashtable con classes en los key y valor de betha en el value"
	systemReturnsCache:=nil. "Guarda los retornos del sistema para calcular el beta"
	percentagesOfChanges:=Dictionary new. "Hashtable con classes en los key y el porcentaje de cambio durante la historia del sistema como value"
	! !

!SpiritModel methodsFor: 'BetaCalculation' stamp: 'SantiagoVidal 7/13/2012 08:55'!
locHistory: aCollection
	locHistory:= aCollection.
	self calculateBetaValues.
	! !

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 7/10/2012 14:40'!
removeFeature: aFeature
	spiritFeatures remove: aFeature ifAbsent:[Error signal: 'error in feature'].! !

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 7/10/2012 14:40'!
removeScenario: aScenario
	spiritScenarios remove: aScenario ifAbsent:[Error signal: 'error in scenario'].! !

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 7/5/2012 16:27'!
spiritFeatures
	^spiritFeatures.! !

!SpiritModel methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 7/5/2012 16:27'!
spiritScenarios
	^spiritScenarios.! !

!SpiritModel methodsFor: 'BetaCalculation' stamp: 'SantiagoVidal 7/14/2012 11:34'!
systemReturns
	(systemReturnsCache=nil)ifTrue: [
		|ret|
		ret:=OrderedCollection new.
		locHistory do:[:aDictionary|
			ret add:(aDictionary keys sum: [:akey| aDictionary at:akey]).
		].
		ret add:((self allClasses select: [:each | each isStub not]) sum: #numberOfLinesOfCode)."Agrego la version actual"
		systemReturnsCache:=self createReturnsFrom:ret.
	].
	^systemReturnsCache. ! !

!SpiritModel methodsFor: 'BetaCalculation' stamp: 'SantiagoVidal 7/16/2012 09:48'!
updateHotSpotsSeverity
	|allHotSpots severityMethod|
	allHotSpots:=OrderedCollection new.
	(self hotSpotsDetected) keys do:[:aKey|
		allHotSpots addAll:((self hotSpotsDetected) at:aKey).
	].
	severityMethod:=SpiritGeneralConfiguration getInstance severityCalculationMethod.
	allHotSpots do:[:aHotspotInstance|
		|famixClass|
		famixClass:=aHotspotInstance famixEntity.
		(famixClass isMethod) ifTrue:[
			famixClass:=famixClass parentType.
		].
		(severityMethod = SpiritGeneralConfiguration BETAMETHOD) ifTrue: [
			aHotspotInstance severityRanking: (bethaValues at:famixClass ifAbsent:[0]).
		] ifFalse: [
			aHotspotInstance severityRanking: (percentagesOfChanges at:famixClass ifAbsent:[0]).
		].
		"Esto esta solo para pruebas"
		aHotspotInstance changesInTheFAMIXEntityDUringLifeTime:((self calculateReturnsFor: famixClass) select: [:aN| (aN=0)not])size.
	].
	! !

Object subclass: #SPAnalizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!SPAnalizer methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:17'!
addHotSpotsInstancesFor: hotSpots ofKind: aKindOfHotSpot in: dictionary 
	hotSpots do:[:anEntity| 
		|array hsInstance| 
		array:=dictionary at: anEntity ifAbsentPut: OrderedCollection new. 
		hsInstance:= HotSpotInstance new. 
		hsInstance kind: aKindOfHotSpot. 
		hsInstance famixEntity: anEntity. array add:  hsInstance.
		hsInstance entitiesAffected: (aKindOfHotSpot new searchAffectedEntities: anEntity).
		hsInstance mainEntities:(aKindOfHotSpot new mainEntities: anEntity).
	].! !

!SPAnalizer methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 2/15/2012 09:59'!
analyse: aHismoModel! !

!SPAnalizer methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 11:31'!
analyseMooseModel: aMoseModel
	|classes methods hotSpots hotspotsByEntity|
	
	classes:=aMoseModel allClasses select: [:each | each isStub not].
	methods:=aMoseModel allMethods select: [:each | each isStub not].
	hotspotsByEntity:=Dictionary new.
	
	hotSpots:=classes select:[:aClass| (BrainClass new) analyze: aClass].
	self addHotSpotsInstancesFor: hotSpots ofKind: BrainClass in: hotspotsByEntity.
	
	hotSpots:=methods select:[:aMethod| (BrainMethod new) analyze: aMethod].
	self addHotSpotsInstancesFor: hotSpots ofKind: BrainMethod in: hotspotsByEntity.
	
	hotSpots:=classes select:[:aClass| (DataClass new) analyze: aClass].
	self addHotSpotsInstancesFor: hotSpots ofKind: DataClass in: hotspotsByEntity.
	
	hotSpots:=methods select:[:aMethod| (DispersedCoupling new) analyze: aMethod].
	self addHotSpotsInstancesFor: hotSpots ofKind: DispersedCoupling in: hotspotsByEntity.
	
	hotSpots:=methods select:[:aMethod| (FeatureEnvy new) analyze: aMethod].
	self addHotSpotsInstancesFor: hotSpots ofKind: FeatureEnvy in: hotspotsByEntity.
	
	hotSpots:=classes select:[:aClass| (GodClass new) analyze: aClass].
	self addHotSpotsInstancesFor: hotSpots ofKind: GodClass in: hotspotsByEntity.
	
	hotSpots:=methods select:[:aMethod| (IntensiveCoupling new) analyze: aMethod].
	self addHotSpotsInstancesFor: hotSpots ofKind: IntensiveCoupling in: hotspotsByEntity.
	
	hotSpots:=classes select:[:aClass| (RefusedParentRequest new) analyze: aClass].
	self addHotSpotsInstancesFor: hotSpots ofKind: RefusedParentRequest in: hotspotsByEntity.
	
	hotSpots:=methods select:[:aMethod| (ShotgunSurgery new) analyze: aMethod].
	self addHotSpotsInstancesFor: hotSpots ofKind: ShotgunSurgery in: hotspotsByEntity.
	
	hotSpots:=classes select:[:aClass| (TraditionBreaker new) analyze: aClass].
	self addHotSpotsInstancesFor: hotSpots ofKind: TraditionBreaker in: hotspotsByEntity.
	
	^hotspotsByEntity.! !

Object subclass: #SPHotSpot
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!SPHotSpot methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:09'!
analyze: anEntity
	self subclassResponsibility! !

!SPHotSpot methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:08'!
mainEntities: anEntity
	self subclassResponsibility.! !

!SPHotSpot methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:09'!
searchAffectedEntities: anEntity
	self subclassResponsibility.! !

SPHotSpot subclass: #SPMultipleHotSpot
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

SPHotSpot subclass: #SPSimpleHotSpot
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

SPSimpleHotSpot subclass: #BrainClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!BrainClass class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:49'!
color
	^Color blue.! !

!BrainClass class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:06'!
kind
	^'BrainClass'! !

!BrainClass class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:47'!
printName
	^'Brain Class'! !

!BrainClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 2/24/2012 14:37'!
analyze: aClass
	|numberOfBrainMethods|
	(GodClass new analyze: aClass) ifTrue:[^false.].
	numberOfBrainMethods:= self numberOfBrainMethods: aClass.
	
	((numberOfBrainMethods >1) & (aClass numberOfLinesOfCode > Threshold LOCClassVeryHigh) & (aClass weightedMethodCount > Threshold WMCVeryHigh)) ifTrue:[^true].
	
	((numberOfBrainMethods==1) & (aClass numberOfLinesOfCode > (2*Threshold LOCClassVeryHigh)) & (aClass weightedMethodCount > (2*Threshold WMCVeryHigh))) ifTrue:[^true].
	
	^false.! !

!BrainClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 12:51'!
mainEntities: aClass
"Devuelve los brainMethods de la clase"
	|ret brainMethod|
	ret:=OrderedCollection new.
	brainMethod:=BrainMethod new.
	
	aClass methodsDo: [:aMethod|
		(brainMethod analyze: aMethod) ifTrue:[ret add:aMethod.].
	].
	^ret.! !

!BrainClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 2/24/2012 10:46'!
numberOfBrainMethods: aClass
	|brainMethod count|
	brainMethod:=BrainMethod new.
	count:=0.
	aClass methodsDo: [:aMethod|
		(brainMethod analyze: aMethod) ifTrue:[count:=count+1.].
	].
	^count.! !

!BrainClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 15:35'!
searchAffectedEntities: aClass
"Son clases muy complejas que tienden a acumular una excesiva cantidad de inteligencia, usualmente en la forma de metodos afectados por BrainMethod"
"La refactorizacion esta dirgida a eliminar los brainMethods"
	^ aClass invokedClasses union: aClass invokingClasses ! !

SPSimpleHotSpot subclass: #BrainMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!BrainMethod class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:49'!
color
	^Color brown.! !

!BrainMethod class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:06'!
kind
	^'BrainMethod'! !

!BrainMethod class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:48'!
printName
	^'Brain Method'! !

!BrainMethod methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 2/29/2012 16:37'!
analyze: aMethod
	((aMethod numberOfLinesOfCode > Threshold LOCVeryHigh) & ("(aMethod hierarchyNestingLevel)"(aMethod cyclomaticComplexity/2) >= Threshold deep) & (aMethod cyclomaticComplexity >= Threshold many) & ((AdditionalMetrics numberOfAccessedVariables: aMethod) >= Threshold sMemCap)) ifTrue:[^true.].
	^false.! !

!BrainMethod methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:04'!
mainEntities: aMethod
"Simplemente devuelve el metodo"
	|ret|
	ret:=OrderedCollection new.
	ret add:aMethod.
	^ret.! !

!BrainMethod methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 15:35'!
searchAffectedEntities: aMethod
"Es un metodo demasiado grande que centraliza la funcionalidad de una clase. Esto lo convierte en dificil de mantener y entender"
"El refactoring es complejo y depende de cada caso. Sin embargo, puede decirse que el peor caso es cuando el metodo debe dividirse y trasladarse a clases invocadas por el metodo y que invocan al metodo"
^ aMethod invokedClasses union: aMethod invokingClasses ! !

SPSimpleHotSpot subclass: #DataClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!DataClass class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:50'!
color
	^Color cyan.! !

!DataClass class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:07'!
kind
	^'DataClass'! !

!DataClass class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:48'!
printName
	^'Data Class'! !

!DataClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 3/1/2012 10:00'!
analyze:aClass
	
	( 
		((AdditionalMetrics weightOfAClass: aClass)<Threshold oneThird) &    
		( 
			(((aClass numberOfAccessorMethods + aClass numberOfPublicAttributes)>Threshold few) &
			(aClass weightedMethodCount < Threshold WMCHigh)) |
			
			(((aClass numberOfAccessorMethods + aClass numberOfPublicAttributes)>Threshold many) &
			(aClass weightedMethodCount < Threshold WMCVeryHigh))
		
		
		)
		
	 )ifTrue:[^true].
	^false. ! !

!DataClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:27'!
mainEntities: aClass
"Devuelve todos los metodos de la clase"
	^aClass methods asOrderedCollection.! !

!DataClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 15:18'!
searchAffectedEntities: aClass
"Los DataClass son clases que guardan informacion pero no tienen funcionalidad compleja. Por esta razon, el behavior asociado a la clase podria estar en otro lugar"
"Las entidades afectadas seran las clases clientes que llaman a la data class. Esto es asi porque metodos de estas clases seran trasladados a la data class o porque los datos guardados en la data class seran repartidos en  los clientes"
	^aClass invokingClasses."Devuelve las clases que invocan a esta clase"! !

SPSimpleHotSpot subclass: #DispersedCoupling
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!DispersedCoupling class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:50'!
color
	^Color green.! !

!DispersedCoupling class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:07'!
kind
	^'DispersedCoupling'! !

!DispersedCoupling class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:48'!
printName
	^'Dispersed Coupling'! !

!DispersedCoupling methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 2/29/2012 15:32'!
analyze: aMethod
	(
		"(aMethod hierarchyNestingLevel > Threshold shallow) & Lo cambie porque no puedo calcular el maxnesting de un metodo"
		((aMethod cyclomaticComplexity/2) > Threshold shallow) & "Esto es lo mas parecido al nesting"
		( 
			(((AdditionalMetrics couplingIntensity: aMethod) > Threshold sMemCap) &
			((AdditionalMetrics couplingDispersion: aMethod)>= Threshold half)) 
			
		)
	) ifTrue:[^true].
	^false.! !

!DispersedCoupling methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:04'!
mainEntities: aMethod
"Simplemente devuelve el metodo"
	|ret|
	ret:=OrderedCollection new.
	ret add:aMethod.
	^ret.! !

!DispersedCoupling methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 16:03'!
searchAffectedEntities: aMethod
"Este es el caso de un metodo que llama a muchos metodos que estan implementados en clases diferentes"
"El refactoring es similar a BrainMethod"
^ aMethod invokedClasses union: aMethod invokingClasses ! !

SPSimpleHotSpot subclass: #FeatureEnvy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!FeatureEnvy class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:50'!
color
	^Color orange.! !

!FeatureEnvy class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:07'!
kind
	^'FeatureEnvy'! !

!FeatureEnvy class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:48'!
printName
	^'Feature Envy'! !

!FeatureEnvy methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 3/1/2012 11:11'!
analyze: aMethod
	(((AdditionalMetrics accessToForeignDataForAMethod: aMethod )> Threshold few) &((AdditionalMetrics foreignDataProvidersForAMethod: aMethod)<=Threshold few) &((AdditionalMetrics localityOfAttributesAccesses: aMethod)<Threshold oneThird))ifTrue:[^true].
	^false.! !

!FeatureEnvy methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:05'!
mainEntities: aMethod
"Simplemente devuelve el metodo"
	|ret|
	ret:=OrderedCollection new.
	ret add:aMethod.
	^ret.! !

!FeatureEnvy methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 15:05'!
searchAffectedEntities: aMethod
"Son metodos que acceden a muchos datos de otras clases a traves de metodos o directamente. Esto podria ser un signo de que el metodo esta mal ubicado y deberia ser movido a otra clase"
"El refactoring propone trasladar el metodo (o parte de el) a la clase con la cual esta mas acoplado. Por esta razon la entidad afectada es la clase con la que esta mas acoplada. Sin embargo el peor caso es que el metodo se parta en otros metodos que se trasladen a cada una de las clases acopladas"
	^aMethod invokedClasses.
! !

SPSimpleHotSpot subclass: #GodClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!GodClass class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:51'!
color
	^Color pink.! !

!GodClass class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:07'!
kind
	^'GodClass'! !

!GodClass class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:48'!
printName
	^'God Class'! !

!GodClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 2/24/2012 11:07'!
analyze: aClass
	(((AdditionalMetrics accessToForeignData: aClass) > Threshold few) & (aClass tightClassCohesion<Threshold  oneThird )& (aClass weightedMethodCount >= Threshold WMCVeryHigh)) ifTrue:[
		^true.
	].
	 ^false.! !

!GodClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:27'!
mainEntities: aClass
"Devuelve todos los metodos de la clase"
	^aClass methods asOrderedCollection.! !

!GodClass methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 14:33'!
searchAffectedEntities: aClass
"GodClass es un code smell en el que una clase hace demasiadas cosas delegando detalles menores a clases triviales y usando informacion de otras clases. SI la clase esta en una parte del sistema que no se modifica entonces NO es un problema"
"Los componentes que afectaria su refactorizacion son las clases que colaboran con la clase. Por esta razon el peor caso es que todas las clases invocadas sean modificadas"
	^aClass invokedClasses.! !

SPSimpleHotSpot subclass: #IntensiveCoupling
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!IntensiveCoupling class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:52'!
color
	^Color yellow.! !

!IntensiveCoupling class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:07'!
kind
	^'IntensiveCoupling'! !

!IntensiveCoupling class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:48'!
printName
	^'Intensive Coupling'! !

!IntensiveCoupling methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 2/29/2012 16:29'!
analyze: aMethod
	(
		"(aMethod hierarchyNestingLevel > Threshold shallow) &  Lo cambie porque no puedo calcular el maxnesting de un metodo"
		((aMethod cyclomaticComplexity/2) > Threshold shallow) & "Esto es lo mas parecido al nesting"
		( 
			(((AdditionalMetrics couplingIntensity: aMethod) > Threshold sMemCap) &
			((AdditionalMetrics couplingDispersion: aMethod)< Threshold half)) |
			
			(((AdditionalMetrics couplingIntensity: aMethod) > Threshold few) &
			((AdditionalMetrics couplingDispersion: aMethod)< Threshold oneQuarter))
			
		)
	) ifTrue:[^true].
	^false.! !

!IntensiveCoupling methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:05'!
mainEntities: aMethod
"Simplemente devuelve el metodo"
	|ret|
	ret:=OrderedCollection new.
	ret add:aMethod.
	^ret.! !

!IntensiveCoupling methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 15:48'!
searchAffectedEntities: aMethod
"Identifica el caso en el que un metodo solo llama metodos de una o unas pocas clases. Ademas, debe haber muchas llamadas a estos metodos."
"La refactorizacion se dara en la clase receptora de los cambios donde posiblemente deban crearse nuevos metodos"
	^aMethod invokedClasses.! !

SPSimpleHotSpot subclass: #RefusedParentRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!RefusedParentRequest class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:52'!
color
	^Color magenta.! !

!RefusedParentRequest class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:07'!
kind
	^'RefusedParentRequest'! !

!RefusedParentRequest class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:49'!
printName
	^'Refused Parent Request'! !

!RefusedParentRequest methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 2/29/2012 11:34'!
analyze: aClass
   "Primero chequear que la clase tenga una superclass (mayor a Object) y que la superclass no sea una interface ni este en una libreria (es decir, esta dentro de allModel clases)"
	|superclass|
	superclass:=aClass superclass.
	(superclass ==nil) ifFalse:[
		(superclass isStub) ifFalse:[
			(superclass isInterface) ifFalse:[
				(
					((((AdditionalMetrics numberOfProtectedMembers: superclass)>Threshold few) &
					((AdditionalMetrics baseClassUsageRatio: aClass) < Threshold oneThird)) |
					((AdditionalMetrics baseClassOverridingRatio: aClass) < Threshold oneThird))
					&
					( 
						(((AdditionalMetrics averageMethodWeight: aClass)> Threshold AMWAvg) |
						(aClass weightedMethodCount >Threshold WMCAvg)) &
						((aClass methods size)>Threshold NOMAvg)
					
					)
				) ifTrue:[^true.].
			].
		].
	].
	^false.! !

!RefusedParentRequest methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:05'!
mainEntities: aClass
"Simplemente devuelve la clase"
	|ret|
	ret:=OrderedCollection new.
	ret add: aClass.
	^ret.! !

!RefusedParentRequest methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 19:21'!
searchAffectedEntities: aClass
|superclasses|
"Es una subclase que no utiliza los metodos de la superclase"
"La refactorizacion es muy compleja y depende de cada caso. Sin embargo puede decirse que el peor caso es cuando debe modificarse todos la cadena de padres de la clase"
	superclasses := OrderedCollection new.
	aClass allSuperclassesDo: [ :aSuperClass | superclasses add: aSuperClass ].
	^superclasses.! !

SPSimpleHotSpot subclass: #ShotgunSurgery
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!ShotgunSurgery class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:52'!
color
	^Color purple.! !

!ShotgunSurgery class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:08'!
kind
	^'ShotgunSurgery'! !

!ShotgunSurgery class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:49'!
printName
	^'Shotgun Surgery'! !

!ShotgunSurgery methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 3/1/2012 10:31'!
analyze: aMethod
	(((AdditionalMetrics changinMethods: aMethod)> Threshold sMemCap) &
	 ( (AdditionalMetrics changingClasses: aMethod)  >Threshold many)
	)ifTrue:[
		(aMethod isConstructor | aMethod isConstant) ifFalse:[
			^true
		].
	].
	^false.! !

!ShotgunSurgery methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:06'!
mainEntities: aMethod
"Simplemente devuelve el metodo"
	|ret|
	ret:=OrderedCollection new.
	ret add:aMethod.
	^ret.! !

!ShotgunSurgery methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 16:09'!
searchAffectedEntities: aMethod
"Este es el caso de un metodo que es invocado por muchos metodos de diferentes clases. Si este metodo cambia habra que realizar muchos cambios pequeos en los callers"
"En general se traslada behavior de los callers a la clase del metodo"
^aMethod invokingClasses.! !

SPSimpleHotSpot subclass: #TraditionBreaker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!

!TraditionBreaker class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 10:53'!
color
	^Color gray.! !

!TraditionBreaker class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/4/2012 12:08'!
kind
	^'TraditionBreaker'! !

!TraditionBreaker class methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/26/2012 11:49'!
printName
	^'Tradition Breaker'! !

!TraditionBreaker methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 2/29/2012 13:47'!
analyze: aClass
	|superclass|
	superclass:=aClass superclass.
	(superclass ==nil) ifFalse:[
		(superclass isStub) ifFalse:[
			(superclass isInterface) ifFalse:[
				(
					(((AdditionalMetrics numberOfAddedServices: aClass)>=Threshold NOMAvg)&
					((AdditionalMetrics percentageOfNewlyAddedServices: aClass)>=Threshold twoThirds)) &
					( (((AdditionalMetrics averageMethodWeight: aClass)>Threshold AMWAvg)|
					   (aClass weightedMethodCount >= Threshold WMCVeryHigh)) &
					   ((aClass methods size) >=Threshold NOMHigh))&
					( ((AdditionalMetrics averageMethodWeight: superclass)>Threshold AMWAvg)& 
					  ((superclass methods size) >(Threshold NOMHigh/2))&
					  (superclass weightedMethodCount >= (Threshold WMCVeryHigh/2)) 	
					)
				) ifTrue:[^true.].
			].
		].
	].
	^false.! !

!TraditionBreaker methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/28/2012 13:08'!
mainEntities: aClass
"Simplemente devuelve la clase"
	|ret|
	ret:=OrderedCollection new.
	ret add: aClass.
	^ret.! !

!TraditionBreaker methodsFor: 'as yet unclassified' stamp: 'SantiagoVidal 6/5/2012 19:17'!
searchAffectedEntities: aClass
|superclasses|
"Es una subclase que solo agrega nueva funcionalidad pero que no especializa demasiado los servicios que hereda"
"La refactorizacion es muy compleja y depende de cada caso. Sin embargo puede decirse que el peor caso es cuando debe modificarse todos la cadena de padres de la clase"
	superclasses := OrderedCollection new.
	aClass allSuperclassesDo: [ :aSuperClass | superclasses add: aSuperClass ].
 ^superclasses.! !

Object subclass: #SpiritHismoModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spirit-Core'!
